from argparse import ArgumentParser

import uvicorn


class CLICommand:
    """通过api远程执行脚本及命令，文档地址: /docs
    """

    @staticmethod
    def add_argments(parser: ArgumentParser):
        add = parser.add_argument
        add("port", help="remote shell外网端口号")

    @staticmethod
    def run(args, parser):
        uvicorn.run("htscf.utils.remote_sh:app", port=int(args.port), host="0.0.0.0")





import socketserver
from multiprocessing import Process, Value, Semaphore, Manager
from os import kill
from signal import SIGKILL
import json


class FlowHandler:
    def __init__(self, DEBUG=False):
        self.value = Value("l", -1)
        self.value2 = Value("l", -1)
        self.signal = Semaphore(1)
        self.data = Manager().dict()
        self.DEBUG = DEBUG

    def handle(self, data):
        pass

    def handleRunning(self, data):
        pass

    def handleExit(self, data):
        pass

    def runFlow(self, signal, value, value2, data):
        """
        执行流程的代码
        """
        try:
            self.startRunningProcess(data)
            self.handle(data)
        except Exception as e:
            print(e)
        finally:
            self.stopRunningProcess(value2)
            FlowHandler.resetValue(value)
            FlowHandler.resetValue(value2)
            signal.release()

    def startRunningProcess(self, data):
        """
        在流程执行时，启动同步监控的代码
        """
        p = Process(target=self.handleRunning, args=[data])
        p.start()
        with self.value2.get_lock():
            if self.DEBUG:
                print(f"running pid:{p.pid}")
            self.value2.value = p.pid

    @staticmethod
    def resetValue(value):
        """重置共享内存值为-1"""
        with value.get_lock():
            value.value = -1

    def stopRunningProcess(self, value2):
        """流程进程结束时，同时也结束同步运行的进程"""
        with value2.get_lock():
            if value2.value != -1:
                try:
                    kill(value2.value, SIGKILL)
                except Exception:
                    print(f"running process(pid:{value2.value}) already exit")
                FlowHandler.resetValue(value2)

    def create(self, value, value2, signal, data):
        """
        用于创建流程，每次只能创建一个，当上一个流程被终止或运行结束时，启动新的流程
        """
        while signal.acquire():
            process = Process(target=self.runFlow, args=[signal, value, value2, data])
            process.start()
            with value.get_lock():
                if self.DEBUG:
                    print(f"flow pid:{process.pid}")
                value.value = process.pid

    def createHandler(self):
        value = self.value
        value2 = self.value2
        signal = self.signal
        data = self.data
        this = self
        p1 = Process(target=self.create, args=[value, value2, signal, data])  # 初始化"创建流程进程"以创建"流程进程"
        p1.start()

        class MyTCPHandler(socketserver.BaseRequestHandler):
            def handle(self):
                self.data = self.request.recv(1024).strip()
                if self.data.decode("utf-8") == str(value.value):
                    with value.get_lock():
                        pid = value.value
                        try:
                            kill(value.value, SIGKILL)
                        except Exception:
                            print(f"Flow process(pid:{value.value}) already exit")
                        value.value = -1
                        this.handleExit(this.data)
                        this.stopRunningProcess(value2)
                        signal.release()  # 释放锁，以是的创建流程的进程激活以创建新的流程进程
                        self.request.sendall(b"OK,killed " + bytes(str(pid), "utf-8"))
                elif self.data.decode("utf-8") == "getpid":
                    pid = value.value
                    self.request.sendall(bytes(str(pid), "utf-8"))

        return MyTCPHandler

    def createServer(self, host, port):
        with socketserver.TCPServer((host, port), self.createHandler()) as server:
            print(f"Server started at tcp://{host}:{port}")
            server.serve_forever()


if __name__ == '__main__':
    import time


    class MyFlowHandler(FlowHandler):
        def __init__(self):

            super().__init__(DEBUG=True)

        def handle(self, data):
            while True:
                time.sleep(1)
                print("hi")

        def handleExit(self, data):
            print("exit 0。。。")

        def handleRunning(self, data):
            while True:
                time.sleep(1)
                print("Running")


    MyFlowHandler().createServer("localhost", 8086)



import re
from pathlib import Path
from subprocess import Popen, PIPE
from typing import Union
from uuid import uuid4

from htscf.db.mongo import connect
from htscf.utils.tools import writeScript, py2cmdline


class Workflow:
    def __init__(self,
                 rootPath: Union[Path, str],
                 stepIds: list[str],
                 dbName: str,
                 stepsCollectionName: str,
                 stepLogCollectionName: str,
                 settingsCollectionName: str,
                 host: str,
                 port: int,
                 flowId: str):
        self.stepsCollection = connect(dbName, stepsCollectionName, host, port)
        self.stepLogCollection = connect(dbName, stepLogCollectionName, host, port)
        self.settingsCollection = connect(dbName, settingsCollectionName, host, port)
        self.rootPath = Path(rootPath)
        self.rootPath.mkdir(exist_ok=True, parents=True)
        self.stepIds = stepIds
        self.flowId = flowId
        self.error = False
        self.logId = None

    def runStep(self, stepId: str):
        if self.logId and self.checkError():
            # 如果没有在上一步的日志中检测到"flow-exit-with-code-0",则退出
            raise Exception("上一步流程出错")
        stepsData = self.stepsCollection.find_one({
            "_id": stepId
        })
        settingsId = f"settings-{uuid4()}"  # 生成随机配置id
        self.settingsCollection.insert_one({
            "_id": settingsId,
            "data": stepsData["settings"]
        })
        scriptPath = writeScript(self.rootPath, stepsData["script"])  # 执行的脚本路径
        popen = Popen([
            stepsData["program"],  # 运行的程序名
            scriptPath,  # 执行的脚本路径
            self.rootPath,  # 运行根目录
            settingsId,  # 数据库中配置文件的ID
            self.logId,  # 日志文件的ID
            self.flowId  # 当前流程的ID
        ], stdout=PIPE, stderr=PIPE)
        popen.wait()
        self.logId = f"{stepId}-{uuid4()}"
        self.stepLogCollection.insert_one({
            "_id": self.logId,
            "data": popen.stdout.read().decode("utf-8"),
            "error": popen.stderr.read().decode("utf-8")
        })

    def checkError(self):
        prevLogData = self.stepLogCollection.find_one({
            "_id": self.logId,
        })["data"]
        if not re.search("flow-exit-with-code-0", prevLogData, re.S):
            return True
        return False

    def run(self):
        while self.stepIds:
            self.runStep(self.stepIds.pop(0))



from argparse import ArgumentParser
from pathlib import Path
from subprocess import Popen, PIPE
from sys import argv


class CLICommand:
    """通过PBS task ID查询当前任务执行路径
    """

    @staticmethod
    def add_argments(parser: ArgumentParser):
        add = parser.add_argument
        add("taskId", help="pbs任务名")

    @staticmethod
    def run(args, parser):
        getPath(args.taskId)


def getPath(taskId):
    popen = Popen(f"qstat -f {taskId}", stdout=PIPE, shell=True)
    popen.wait()
    lines = popen.stdout.read().decode("utf-8").split("\n")[1:]
    arr = []
    while lines:
        line = lines.pop(0)
        if line.startswith("\t") or line.endswith(",") or line.startswith("}"):
            line = line.strip()
            arr[-1] += line
        else:
            line = line.strip()
            if line:
                arr.append(line)
    config = {}
    for i in arr:
        key, value = i.split("=", maxsplit=1)
        key = key.strip()
        value = value.strip()
        config[key] = value
    outputPath = config["Output_Path"].split(":", maxsplit=1)[1]
    rootPath = Path(outputPath).parent
    print(rootPath)



from argparse import ArgumentParser
from getpass import getpass
import requests as rq
import socket
import textwrap
from re import match
import json


class CLICommand:
    """苏大校园网登录脚本
    """

    @staticmethod
    def add_argments(parser: ArgumentParser):
        pass

    @staticmethod
    def run(args, parser):
        print("--苏大校园网登录脚本--")
        sid = input("请输入学号:")
        pswd = getpass("请输入密码:")
        login(sid, pswd)


def login(sid, pswd):
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    mac = 000000000000
    url = f"http://10.9.1.3:801/eportal/?c=Portal&a=login&callback=dr1003&login_method=1&user_account=%2Cb%2C{sid}" \
          f"&user_password={pswd}&wlan_user_ip={ip}&wlan_user_ipv6=&wlan_user_mac={mac}&wlan_ac_ip=" \
          f"&wlan_ac_name=&jsVersion=3.3.3&v=8918"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.101 Safari/537.36"
    }
    response = rq.get(url, headers=headers)
    print(parse(response.text))
    response.close()


def parse(s):
    rt = match(r".*?\((\{.*?})\)", s)
    data = rt.group(1)
    data = json.loads(data)
    return textwrap.dedent(f"""
    ------------------------
    msg: {data["msg"]}
    code: {data["result"]}
    ------------------------
    """).strip()



import sys
from datetime import datetime as __dt
from pathlib import Path
from subprocess import Popen, PIPE
from os.path import exists
import platform


def qsub(nodes=1, cores=28, python_script=sys.argv[0]):
    """
    用于python脚本提交
    :param python_script: 本脚本执行路径
    :param nodes:所用节点数，default:1
    :param cores:每个节点所用核数,default:28
    """
    QSUB_PATH = "qsub.sh"
    if platform.system() == 'Windows':
        return
    if not python_script:
        raise Exception("Please specify python script name!")
    if not exists("run"):
        qsub_data_lines = """
#!/bin/bash
#!/bin/env bash
#PBS -N zh
#PBS -l nodes=1:ppn=28
#PBS -q batch
#PBS -V
cd "${PBS_O_WORKDIR}" || exit
run() {
  local arg
  while getopts "p:" arg
  do
    case $arg in
      p)python "$OPTARG";;
      ?)echo "No such command!";;
    esac
  done
}
run -p "$path"
        """.strip().split("\n")
        for line in qsub_data_lines:
            if line.startswith("#PBS -l"):
                index = qsub_data_lines.index(line)
                qsub_data_lines[index] = f"#PBS -l nodes={nodes}:ppn={cores}"
        with open(QSUB_PATH, "w+") as sh:
            for line in qsub_data_lines:
                sh.write(line + "\n")
        popen = Popen(f'qsub -v path="{python_script}" {QSUB_PATH}', shell=True, stdout=PIPE, stderr=PIPE)
        popen.wait()
        if platform.system() == 'Windows':
            info = popen.stdout.read().decode("gbk")
            error = popen.stderr.read().decode("gbk")
        else:
            info = popen.stdout.read().decode("utf-8")
            error = popen.stderr.read().decode("utf-8")
        print("任务ID:", info, end="")
        with open("run", "a+") as fd:
            fd.write(info)
            fd.write(error)
        exit()
    else:
        with open("run", "a+") as f1:
            f1.write(f"start running! {__dt.now()}")


if __name__ == '__main__':
    qsub(1, 20, "test.py")
